/**
	Webana common functions

	Jan Raszyk
	2007
*/


#include <string>
#include <iostream>
#include <sstream>
#include <map>
#include <vector>

using namespace std;

namespace wbn {

bool WildCardMatch(const char* mask, const char* string)
{
	// special case speed-up: "*" always matches
	if (mask[0] == '*' && mask[1] == '\0') return true;

    if (*string == '\0')
    {
        if (*mask == '\0') return true;
        if (*mask == '?') return false;
        if (*mask == '*') return WildCardMatch(mask+1, string);
        return false;
    }

    switch (*mask) {
    case '?': return WildCardMatch(mask+1, string+1);
    case '*': return (
        WildCardMatch(mask+1, string) ||
        WildCardMatch(mask, string+1) ||
        WildCardMatch(mask+1, string+1)
    );
    case '\0': return *string == '\0';
    default: return (*mask == *string) && WildCardMatch(mask+1, string+1);
    }
}

bool WCVectorMatch(const vector<string> & masks, const string & str)
{
	vector<string>::const_iterator ci;
	for (ci = masks.begin(); ci != masks.end(); ci++)
	{
		if (!WildCardMatch(ci->c_str(), str.c_str()))
			return false;
	}

	return true;
}

bool IsHTTP(const string & url)
{
    if (url.length() < 8) {
        cerr << "Too short URL" << endl;
        return false;
    }
    
    if (url.substr(0, 7).compare("http://"))
    {
        cout << "Only HTTP Supported" << endl;
        return false;
    }
    else
        return true;
}

string GetHostFromURL(const string & url)
{
    if (!IsHTTP(url)) return "";
    int slashpos = url.find("/", 7);
    if (slashpos)
    {
        return url.substr(7, slashpos - 7);
    }
    else
    {
        return url.substr(7, url.length() - 7);
    }
}

string GetPathFromURL(const string & url)
{
    if (!IsHTTP(url)) return "";
    
    int slashpos = url.find("/", 7);
    if (slashpos < 0) return "/";
    return url.substr(slashpos);
}

bool IsLinkToToplevel(const string & url, const string & baseurl)
{
	// check if it does not contain any relative path
	if (GetPathFromURL(url).compare(string("/")))
		return false;

	// now we need to check if baseurl starts with the same string
	if (GetHostFromURL(url).compare(GetHostFromURL(baseurl)))
		return false;

	return true;
}

// "http://www.example.com/dir1/file.php", "/this/dir/file.html"
// will return: "http://www.example.com/this/dir/file.html"
string GetAbsoluteURL(const string & base, const string & relative)
{
	string::size_type sp, cp, lsp;
	string raw_base;

	if (relative.find("http://") == 0) return relative; // it already is absolute path
	if (base.find("http://") != 0) return "";	// base path is not base at all

	if (relative.empty()) return base;

	sp = relative.find("/");
	cp = relative.find(":");

	if (cp != string::npos && (sp < cp || sp == string::npos))
		// looks like a protocol is specified and it's not HTTP
		return "";

	lsp = base.find_last_of("/");
	// if last slash in base path is the last slash in 'http://', add it
	// otherwise raw base path is base path until the last slash
	raw_base = lsp == 6 ? base + "/" : base.substr(0, lsp + 1);
	
	if (relative[0] == '/')
	{
		return raw_base.substr(0, raw_base.find_first_of("/", 7)) + relative;
	}
	else
	{
		return raw_base + relative;
	}
}

};
