/**
    Webana Downloader module

	Responsible for downloading web pages via HTTP
    Jan Raszyk, April 2007
*/

#include <winsock2.h>
#include <process.h>
#include <string>
#include <iostream>
#include <sstream>
#include <map>
#include <vector>
#include "html_parse.h"
#include "tcpclient.h"
#include "downloader.h"
#include "common.h"

using namespace std;

namespace wbn {

HtmlPageGetter::HtmlPageGetter()
{
	tcpclient = new TCPClient;

    state = LS_IDLE;

    leech_thread = 0;

    headers.inited = false;

	dom = 0;
}

HtmlPageGetter::~HtmlPageGetter()
{
	delete tcpclient;
	if (dom) delete dom;
}

downloader_state HtmlPageGetter::GetState() const { return this->state; }
bool HtmlPageGetter::Finished() const
{
	return !this->leeching;

	// --

	DWORD ec;
	BOOL r = GetExitCodeThread(thread_handle, &ec);

	if (!r) return true;
	if (ec == STILL_ACTIVE) return false;

	return true;
}

int HtmlPageGetter::GetDepth() const { return _depth; }


void HTTPHeaders::Add(const string & name, const string & value) { headers[name] = value; }
bool HTTPHeaders::Exists(const string & name) const {
    return headers.find(name) != headers.end();
}
string HTTPHeaders::ValueOf(const string & name) const
{
    map<string, string>::const_iterator ci;
    ci = headers.find(name);
    if (ci != headers.end())    return ci->second;
    else                        return "";
}
int HTTPHeaders::StatusCode(void) const { return statuscode; }

void HTTPHeaders::ReadFromStream(stringstream & ss)
{
    string s, h, b;
    int l = 0;
    int c;

    while (1) {
        l++;
        if (ss.eof()) { inited = true; break; }

		getline(ss, s);

        if (l == 1)
        {
			stringstream ss2(s);
            string httpv, status, reason;
            ss2 >> httpv >> status >> reason;
            statuscode = atoi(status.c_str());
            continue;
        }

		if (s.empty() || s.c_str()[0] == '\n' || s.c_str()[0] == '\r')
        { 
            inited = true;
            break;
        }

        if (s.find(":", 0) < 0) continue;
        istringstream is(s);

        getline(is, h, ':');
        if (is.eof()) break;
        if ((c = is.get()) != ' ') is.putback(c); // skip the space
        getline(is, b, '\r');

        Add(h,b);
    }
}

int HtmlPageGetter::SendGETHeader(const string & hostname, const string & pathname)
{
    string buf;
    
    buf  = "GET " + pathname + " HTTP/1.1\r\n";
    buf += "Host: " + hostname + "\r\n";
    buf.append("Accept: text/html, application/xhtml+xml, application/xml+xhtml, application/xhtml, application/xml\r\n");
    buf.append("User-Agent: WebAnalyzer 1.0 (http://johnny.wz.cz/web-analyzer/)\r\n");
    buf.append("\r\n");
    
#if 0
    cout << "--Sending--" << endl;
    cout << buf;
    cout << "----" << endl;
#endif

	tcpclient->Send(buf.c_str(), buf.length());

	if (!tcpclient->Good())
		return 0;
	else
		return 1;
}

void HtmlPageGetter::GetChunked()
{
    string cl, te;
    cl = headers.ValueOf("Content-Length");
    te = headers.ValueOf("Transfer-Encoding");

    if (!cl.empty())
    {
        transfer_type = TR_TOTAL_LENGTH;
        stringstream ss(cl);
        ss >> content_length;
    }
    else if (!te.empty())
    {   
        transfer_type = TR_CHUNKED;
    }
    else 
    {
        transfer_type = TR_UNKNOWN;
    }

}

// HTTP data parser
void HtmlPageGetter::ReadHTTPChunk(const string & input)
{
    string::size_type pos;
    string s = body_buffer + input;

	body_buffer = "";

    if (state == LS_HEAD)
    {
        pos = s.find("\r\n\r\n");
        if (pos == string::npos)
        {
            header_buffer << s;
        }
        else
        {
            header_buffer << s.substr(0, pos);
            headers.ReadFromStream(header_buffer);
			// todo - check for 100
			if (headers.inited && (headers.StatusCode() > 299 || headers.StatusCode() < 200)) {
				state = LS_ERROR;
				return;
			}
            s = s.substr(pos+4);
            state = LS_BODY;
            GetChunked();   // will store content_length in case transfer_type is TR_TOTAL_LENGTH
            if (transfer_type == TR_CHUNKED)
            {
                pos = s.find("\r\n");
                nextchunk = (int) strtol(s.c_str(), 0, 16);
                if (pos != string::npos)
                {
                    s = s.substr(pos+2);
                } // else should not happen!
            }
        }
    }
    
    if (state == LS_BODY)
    {
        if (transfer_type == TR_CHUNKED)
        {
            do
            {
                if (nextchunk > (int) s.length())
                {	// it's safe to eat it all
                    htmltext += s;
                    nextchunk -= s.length();
                    s = "";
                }
                else
                {
                    htmltext += s.substr(0, nextchunk);
                    s = s.substr(nextchunk);
					nextchunk = 0;
                    if (s.length() < 2)
                    {
                        body_buffer = s;
						s = "";
                    }
                    else 
                    {
                        s = s.substr(2);
						pos = s.find("\r\n");
						if (pos != string::npos)
						{   // there is enough data in the buffer to get the length of next chunk
                            nextchunk = (int) strtol(s.c_str(), 0, 16);
                            if (!nextchunk)
                            {
                                state = LS_READY; return;
                            }
                            s = s.substr(pos+2);
                        }
                        else
                        {
                            body_buffer = s;
                            s = "";
                        }
                    }
                }
            } while (!s.empty());
        }
        else if (transfer_type == TR_TOTAL_LENGTH)
        {
            htmltext.append(s);
            content_length -= s.length();
			if (content_length <= 0)
				state = LS_READY;
        }
        else
        {
            htmltext.append(s);
        }
    }    
}

int HtmlPageGetter::ReadTCP(void)
{
    char recvbuf[8*1024];
    int i;

    state = LS_HEAD;

    do {
        i = tcpclient->Recv(recvbuf, sizeof(recvbuf)-1);

		if		(!tcpclient->Good())	state = LS_ERROR;
		else if (tcpclient->Closed())	state = LS_READY;
		else
		{
			recvbuf[i] = '\0';
			if (i > 0) ReadHTTPChunk(string(recvbuf));
		}
	} while (i>0 && state != LS_READY && state != LS_ERROR);

    return 1;
}

void HtmlPageGetter::Process(void)
{
    dom = new HTML_DOMTreeCSSDump(htmltext.c_str(), csstext);
	dom->Parse();
}

void HtmlPageGetter::LeechPage() {
#if 0
	cout << "Download of " << url << " has started..." << endl;
#endif

	thread_handle = GetCurrentThread();
    if (!IsHTTP(url)) { state = LS_ERROR; return; }
    string hostname = GetHostFromURL(url);
    string pathname = GetPathFromURL(url);
    
	if (hostname.empty()) { state = LS_ERROR; return; }

	tcpclient->Connect(hostname, 80);
	if (!tcpclient->Good() || tcpclient->Closed()) { state = LS_ERROR; return; }

    state = LS_CONNECTED;

    if (!SendGETHeader(hostname, pathname)) { state = LS_ERROR; return; }

    if (!ReadTCP()) { state = LS_ERROR; return; }

    Process();

    state = LS_READY;

    return;
}

static unsigned __clrcall SLeechPage(void * par)
/* DWORD WINAPI _LeechPage(LPVOID par) */
{
    HtmlPageGetter * thishpg = (HtmlPageGetter *) par;
    thishpg->LeechPage();
	thishpg->leeching = false;
    return 1;
}

void HtmlPageGetter::StartDownload(const std::string & URL, int depth)
{
    unsigned int ta;

	if (state == LS_ERROR) return;

    this->url = URL;
    this->_depth = depth;
	this->leeching = true;
	leech_thread = _beginthreadex(NULL, 0, SLeechPage, this, 0, &ta);
}

int HtmlPageGetter::GetLinks(std::vector<std::string> & b) const
{
    vector<HTMLElement *>::const_iterator ei;
    map<string, string>::const_iterator ai;
	HTMLElement *hep;
    const string anchor("a");

	int i = 0;

	if (!dom) return 0;

	int http_status = headers.StatusCode();
	if (http_status >= 300 && http_status <= 399)
	{
		string s = headers.ValueOf("Location");
		if (!s.empty()) {
			b.push_back(s);
			return 1;
		}
		return 0;
	}

	if (http_status >= 300) return 0;

    for (ei = dom->elems.begin(); ei != dom->elems.end(); ei++)
    {
		hep = *ei;
        if (hep->name == anchor)
        {
            ai = hep->attrs.find("href");
            if (ai != hep->attrs.end())
            {
				i++;
                b.push_back(ai->second);
            }
        }
    }

	return i;
}

}; // namespace wbn

