/**
    Webana Downloader module

	Responsible for downloading web pages via HTTP
    Jan Raszyk, April 2007
*/
#pragma once

namespace wbn {

typedef enum { LS_IDLE, LS_CONNECTED, LS_HEAD, LS_BODY, LS_ERROR, LS_READY } downloader_state;
typedef enum { TR_UNKNOWN, TR_TOTAL_LENGTH, TR_CHUNKED } transfer_type_t;
typedef std::map<std::string, std::string>::const_iterator headers_iter;

// represents HTTP headers sent by a server, received on the client side
class HTTPHeaders
{
private:
    std::map<std::string, std::string>
                        headers;				// header and it's value
    int                 statuscode;				// HTTP status code returned by the server
    void                Add(const std::string &, const std::string &);	// adds given header to the structure

public:
    int                 StatusCode(void) const;	// returns statuscode
    bool                inited;					// have headers been already read?
    void                ReadFromStream(std::stringstream &);	// reads HTTP headers from given stream
    bool                Exists(const std::string &) const;	// does given header exist?
    std::string         ValueOf(const std::string &) const;	// return value of the given header info
};

// ensures TCP connection, HTTP transfer, HTML parsing
// receives 1 HTML page from the internet and builds Document Object Model
class HtmlPageGetter
{
private:
	TCPClient*			tcpclient;				// tcp client connection
    uintptr_t           leech_thread;			// pointer to the downloader thread (Visual C++)
	HANDLE				thread_handle;			// handle of the downloader thread (WinAPI)
    int                 SendGETHeader(const std::string & hostname, const std::string & pathname);	
												// sends GET reques
    int                 ReadTCP(void);			// reads data from server
    downloader_state    state;					// current downloading state
    void                Process(void);			// builds Document Object Model
    std::stringstream   header_buffer;			// buffer for parsing purposes
    std::string         body_buffer;			// buffer for parsing purposes
    int                 _depth;					// relative crawling depth of this page
    void                ReadHTTPChunk(const std::string & s); // process a piece of data received from the server
    void                GetChunked(void);		// checks how is the length of the content specified
    transfer_type_t     transfer_type;			// content length specification
    int                 nextchunk;				// remaining length of the currently read chunk of data
    int                 content_length;			// remaining length of the document body

public:
    /* constructor */   HtmlPageGetter();
    /* destructor */    ~HtmlPageGetter();
    std::string         htmltext;				// HTML content of the document
	std::string			csstext;				// CSS content
	wbn::HTML_DOMTree   *dom;					// Document Object Model retrieved from the HTML content
    std::string         url;					// URL of the page associated with current instance
	HTTPHeaders         headers;				// HTTP headers of the current page
    void                StartDownload(const std::string & URL, int);
		// starts downloading thread of the given page
		// second parameter is the relative crawling depth of the given page

    void				LeechPage();			// main downloader function
	int					GetLinks(std::vector<std::string> &) const;
		// reads links from the DOM and writes them into given output vector of strings
	bool				leeching;				// is the download thread active?
    bool				Finished() const;		// has the download finished?
	downloader_state    GetState() const;		// result of the download
    int                 GetDepth() const;		// relative crawling depth of the current page
};

}; // namespace wbn
