/**

	Web Analyzer HTML Parser module

	Goes through the HTML input and builds Document Object Model structure
	Jan Raszyk, 2007

*/

#ifndef __HTML_PARSE_H__
#define __HTML_PARSE_H__

namespace wbn {

// helper class, not used in the DOM structure
class HTMLAttr
{
public:
    HTMLAttr(const char *, const char *);
    const char* name;
    const char* val;
};

// representation of one HTML element
struct HTMLElement 
{
    std::string             name;
    std::map<std::string, std::string>    attrs;
};

typedef std::vector<HTMLElement *> elementslist;

// represents Document Object Model Tree
// in fact it's not a tree, we use simplified structure - a list of elements
class HTML_DOMTree
{
protected:
	const char* text;
	HTMLElement* GetElement(void);
	virtual void ElementFound(const HTMLElement & e);
	virtual void ElementOver();
	virtual void SkipTillElStart();
	virtual void SkipCommentContent();
	virtual void SkipCData(void);

public:
	HTML_DOMTree(const char* html);
	// main parser function, read html data from given input string and creates DOM
	void Parse();
	~HTML_DOMTree();
	// list of elements
    elementslist    elems;
	// doctype declaration, or it's important part
    std::string doctype;
	// did the document contain XML declaration on the beginning?
    bool    xmldecl;
};

// adds functionality to dump text from within the <style> element into given string
class HTML_DOMTreeCSSDump : public HTML_DOMTree
{
	std::string & cssexportstring;
	bool in_css_section;
	virtual void HTML_DOMTreeCSSDump::SkipTillElStart();
	virtual void ElementFound(const HTMLElement & e);
	virtual void ElementOver();
	virtual void SkipCommentContent();
	virtual void SkipCData(void);

public:

	HTML_DOMTreeCSSDump(const char* html, std::string &);
};

// debug output function, prints DOM in human-readable form
std::ostream & operator << (std::ostream & o, const HTML_DOMTree & t);

};

#endif // __HTML_PARSE_H__
