/**
	
	Site stats module

	Builds a set of information about page from given Document Object Model (DOM) object
	author: Jan Raszyk
	May 2007

*/

#ifndef __SITESTATS_H__
#define __SITESTATS_H__

namespace wbn {

typedef enum { PSV_FORMATTED, PSV_AVERAGE, PSV_SEMANTIC_ONLY } page_semantic_value;

// vector of types of images used on web pages
class ImageTypeCounts {
public:
	ImageTypeCounts();
	unsigned jpg_images, png_images, gif_images, other_images;
	ImageTypeCounts operator += (const ImageTypeCounts &);
};

// represents information about single HTML page
class PageStats
{
public:
	PageStats(const HTML_DOMTree &);	// initializes the structure from given DOM, the only constructor available
	std::string doctype;				// doctype declaration
	unsigned format_els;				// amount of formatting elements like b, i, center, font, ...
	unsigned semantic_els;				// amount of semantic value elements like dl, kbd, var, heading, ...
	page_semantic_value semantic_value;	// result page category according to amount of formatting and semantic elements
	bool xml_decl;						// does page have XML declaration?
	bool xml_feed;						// does page have XML (possibly RSS) feed linked?
	bool javascript;					// is javascript included?
	bool stylesheets;					// is stylesheet included?
	bool flash;							// is flash included?
	bool homepagelink;					// is there a link to the homepage present?
	ImageTypeCounts image_types;		// amounts of types of images
}; // PageStats

// gathers information about set of HTML pages
// structure entries are equivalent to PageStats, 
// represent amount of pages that had appropriate PageStats information set to true
class SiteStats
{
private:
	std::map<std::string, unsigned *>	serialize_map;

public:
	unsigned formatted_pages, average_pages, semantical_pages, total_pages;
	unsigned xml_declarations;
	unsigned stylesheeted;
	unsigned javascripted;
	unsigned xmlfeeded;
	unsigned withflash;
	unsigned homepagelinked;
	ImageTypeCounts image_types;
	std::map<std::string, unsigned> doctypes;

	// constructor, initializes structure
	SiteStats();
	
	// adds single page stats to global stats, main function
	SiteStats operator += (const PageStats &);

	// writes content of this structure in HTML format
	friend std::ostream & operator << (std::ostream & o, const SiteStats & c);

	// serialization functions:
	// dumps content of the structure in Web Analyzer project file format
	std::ostream & ProjectDump(std::ostream & o) const;
	// reads the structure content from input stream
	friend std::istream & operator >> (std::istream & i, SiteStats & s);
}; // SiteStats

}; // namespace

#endif // __SITESTATS_H__
