/**
    TCP client module

	Responsible for connecting via TCP and sending/receiving data
    Jan Raszyk, April 2007
*/

#include <winsock2.h>
#include <windows.h>
#include <string>
#include <iostream>
#include "tcpclient.h"

using namespace std;

namespace wbn {

int StartWSA(void)
{
    WSADATA wsaData;
    WORD version;
    int error;

    version = MAKEWORD( 1, 1 );

    error = WSAStartup( version, &wsaData );

    if ( error != 0 )
    {
        cerr << "Cannot start WSA" << endl;
        return -1;
    }

    /* check for correct version
    if ( LOBYTE( wsaData.wVersion ) != 2 ||
         HIBYTE( wsaData.wVersion ) != 0 )
    {
        cerr << "Incorrent WSA version" << endl;
        WSACleanup();
        return -1;
    }
	*/

    return 0;
}

TCPClient::TCPClient() : 
	good(true), closed(true), bytes_in(0), bytes_out(0)
{
    /* WinSock has been initialized */
	sock = socket( AF_INET, SOCK_STREAM, IPPROTO_TCP );

	if (sock == INVALID_SOCKET)
		good = false;
}

TCPClient::~TCPClient()
{
	closesocket( sock );
}

bool TCPClient::Good(void)				const { return good; }
bool TCPClient::Closed(void)			const { return closed; }
unsigned TCPClient::Transferred(void)	const { return bytes_in+bytes_out; }

void TCPClient::Connect(const string & hostname, int port)
{
    struct hostent *host;
    struct sockaddr_in sin;
	int conret;

    host = gethostbyname( hostname.c_str() );

    if (!host)
    {
        good = false;
        return;
    }

    memset(&sin, 0, sizeof(sin));
    sin.sin_family = AF_INET;
    // sin.sin_addr.s_addr = ((struct in_addr *)(host->h_addr))->s_addr;
	memcpy(&(sin.sin_addr), host->h_addr, host->h_length);
    sin.sin_port = htons(port);

	conret = connect(sock, (SOCKADDR *) &sin, sizeof(sin));
	if (conret == SOCKET_ERROR)
    {
        good = false;
        return;
    }

    FD_ZERO(&readfds);
    FD_SET(sock, &readfds);

	closed = false;
    return;
}

int TCPClient::Recv(char* data, int size)
{
	if (!good || closed) return 0;

	timeval tv;
	int rv;
    
    // timeout 10.5 s
    tv.tv_sec = 10;
    tv.tv_usec = 500000;

	rv = select(sock + 1, &readfds, NULL, NULL, &tv);

	if (rv == SOCKET_ERROR)
    {
        good = false;
		return 0;
    }
    else if (rv == 0)
    {
        closed = true;
        return 0;
    }
	else
	{
		int i = recv(sock, data, size, 0);
		bytes_in += i;
		return i;
	}
}

void TCPClient::Send(const char* data, int size)
{
	if (!good || closed) return;

	bytes_out += size;
	send(sock, data, size, 0);
}

};	// namespace wbn
